#include "MarkdownComponents.h"

#include "StringUtils.h"

MarkdownTextSpan::Type MarkdownTextSpan::getType() const
{
    return Type::TEXT_SPAN;
}

MarkdownParagraph::Type MarkdownParagraph::getType() const
{
    return Type::PARAGRAPH;
}

void MarkdownParagraph::addChild(std::unique_ptr<MarkdownInlineElement> child)
{
    mChildren.push_back(std::move(child));
}

std::size_t MarkdownParagraph::getNumChildren() const
{
    return mChildren.size();
}

MarkdownInlineElement* MarkdownParagraph::getChild(std::size_t idx) const
{
    return mChildren[idx].get();
}

MarkdownBulletItem::Type MarkdownBulletItem::getType() const
{
    return Type::BULLET_ITEM;
}

MarkdownBulletList::Type MarkdownBulletList::getType() const
{
    return Type::BULLET_LIST;
}

void MarkdownBulletList::addChild(std::unique_ptr<MarkdownBulletItem> child)
{
    mChildren.push_back(std::move(child));
}

std::size_t MarkdownBulletList::getNumChildren() const
{
    return mChildren.size();
}

MarkdownBulletItem* MarkdownBulletList::getChild(std::size_t idx) const
{
    return mChildren[idx].get();
}

MarkdownHeading::MarkdownHeading(unsigned level)
    : mLevel(level)
{

}

MarkdownHeading::Type MarkdownHeading::getType() const
{
    return Type::HEADING;
}

unsigned MarkdownHeading::getLevel() const
{
    return mLevel;
}

MarkdownInlineQuote::Type MarkdownInlineQuote::getType() const
{
    return Type::INLINE_QUOTE;
}

MarkdownLink::MarkdownLink(const std::string& target)
    : mTarget(target)
{

}

const std::string& MarkdownLink::getTarget() const
{
    return mTarget;
}

MarkdownLink::Type MarkdownLink::getType() const
{
    return Type::LINK;
}

void MarkdownLink::doFieldSubstitution(Type elementType, const std::string& searchPhrase, const std::string& replacementPhrase)
{
    if (elementType == Type::LINK)
    {
        mTarget = StringUtils::replaceWith(mTarget, searchPhrase, replacementPhrase);
    }
}

MarkdownImage::MarkdownImage(const std::string& source, const std::string& alt)
    : mSource(source),
      mAlt(alt)
{

}

MarkdownImage::Type MarkdownImage::getType() const
{
    return Type::IMAGE;
}

const std::string& MarkdownImage::getSource() const
{
    return mSource;
}

const std::string& MarkdownImage::getAlt() const
{
    return mAlt;
}

MarkdownMultilineQuote::MarkdownMultilineQuote(const std::string& tag)
    : mTag(tag)
{

}

MarkdownMultilineQuote::Type MarkdownMultilineQuote::getType() const
{
    return Type::MULTILINE_QUOTE;
}
