#pragma once

#include <memory>
#include <string>

class MarkdownDocument;
class MarkdownParagraph;
class MarkdownBulletList;

class MarkdownParser
{
    enum class DocumentState
    {
        NONE,
        IN_MULTILINEQUOTE,
        IN_BULLETS
    };

    enum class LineState
    {
        NONE,
        IN_HEADING,
        IN_INLINEQUOTE,
        IN_MULTILINE_TAG,
        IN_LINK_TAG,
        AWAITING_LINK_BODY,
        IN_LINK_BODY,
        AWAITING_IMG_TAG,
        IN_IMG_TAG,
        AWAITING_IMG_BODY,
        IN_IMG_BODY,
        IN_BULLETS
    };

public:
    MarkdownParser();

    ~MarkdownParser();

    std::unique_ptr<MarkdownDocument> run(const std::string& content);

private:
    void processLine();

    void onMultilineQuote();
    void onInlineQuote();
    void onHeading(unsigned level);
    void onLink();
    void onImage();

    void onEmptyLine();
    void onNewParagraph();

    void onBulletItem();

    void onTextSpan();

    std::pair<unsigned, bool> onTick(unsigned tickCount);

    std::string mWorkingLine;
    std::string mLineContent;
    std::string mDocumentContent;

    std::string mWorkingTag;

    LineState mLineState {LineState::NONE};
    DocumentState mDocumentState {DocumentState::NONE};

    std::unique_ptr<MarkdownParagraph> mWorkingParagraph;
    std::unique_ptr<MarkdownBulletList> mWorkingBulletList;

    std::unique_ptr<MarkdownDocument> mMarkdownDocument;
};
